@@ -1,14 +1,13 @@
 package CPAN;
 use vars qw{$META $Signal $Cwd $End $Suppress_readline};
 
-$VERSION = '1.07';
+$VERSION = '1.10';
 
-# $Id: CPAN.pm,v 1.87 1996/12/23 03:58:42 k Exp $
+# $Id: CPAN.pm,v 1.99 1997/01/21 00:58:02 k Exp $
 
-# my $version = substr q$Revision: 1.87 $, 10; # only used during development
+# my $version = substr q$Revision: 1.99 $, 10; # only used during development
 
-BEGIN {require 5.003;}
-require UNIVERSAL if $] == 5.003;
+use 5.00315;
 
 use Carp ();
 use Config ();
@@ -59,7 +58,10 @@ $META ||= new CPAN;                 # In case we reeval ourselves we
 
 CPAN::Config->load;
 
-@EXPORT = qw(autobundle bundle expand force install make recompile shell test clean);
+@EXPORT = qw( 
+	     autobundle bundle expand force get
+	     install make readme recompile shell test clean
+	    );
 
 
 
@@ -170,7 +172,11 @@ or
     $self->{LOCK} = $lockfile;
     $fh->close;
     $SIG{'TERM'} = sub { &cleanup; die "Got SIGTERM, leaving"; };
-    $SIG{'INT'} = sub { &cleanup, die "Got a second SIGINT" if $Signal; $Signal = 1; };
+    $SIG{'INT'} = sub {
+	my $s = $Signal == 2 ? "a second" : "another";
+	&cleanup, die "Got $s SIGINT" if $Signal;
+	$Signal = 1;
+    };
     $SIG{'__DIE__'} = \&cleanup;
     print STDERR "Signal handler set.\n" unless $CPAN::Config->{'inhibit_startup_message'};
 }
@@ -222,7 +228,8 @@ sub hasMD5 {
     } elsif (not defined $self->{'hasMD5'}) {
 	eval {require MD5;};
 	if ($@) {
-	    print "MD5 security checks disabled because MD5 not installed. Please consider installing MD5\n";
+	    print "MD5 security checks disabled because MD5 not installed.
+  Please consider installing MD5\n";
 	    $self->{'hasMD5'} = 0;
 	} else {
 	    $self->{'hasMD5'}++;
@@ -314,7 +321,7 @@ Readline support $rl_avail
 	    my(@line);
 	    eval { @line = Text::ParseWords::shellwords($_) };
 	    warn($@), next if $@;
-	    $CPAN::META->debug("line[".join(":",@line)."]");
+	    $CPAN::META->debug("line[".join(":",@line)."]") if $CPAN::DEBUG;
 	    my $command = shift @line;
 	    eval { CPAN::Shell->$command(@line) };
 	    warn $@ if $@;
@@ -327,7 +334,7 @@ Readline support $rl_avail
 }
 
 package CPAN::Shell;
-use vars qw($AUTOLOAD);
+use vars qw($AUTOLOAD $redef);
 @CPAN::Shell::ISA = qw(CPAN::Debug);
 
 # private function ro re-eval this module (handy during development)
@@ -355,12 +362,13 @@ i         none                    anything of above
 
 r          as             reinstall recommendations
 u          above          uninstalled distributions
-See manpage for autobundle, recompile, force, etc.
+See manpage for autobundle, recompile, force, look, etc.
 
-make      modules,        make
-test      dists, bundles, make test (implies make)
-install   "r" or "u"      make install (implies test)
-clean                     make clean
+make                      make
+test      modules,        make test (implies make)
+install   dists, bundles, make install (implies test)
+clean     "r" or "u"      make clean
+readme                    display the README file
 
 reload    index|cpan    load most recent indices/CPAN.pm
 h or ?                  display this menu
@@ -460,7 +468,7 @@ sub o {
 			next unless lc($_) eq lc($what);
 			$CPAN::DEBUG |= $CPAN::DEBUG{$_};
 		    }
-		    print "unknown argument $what\n";
+		    print "unknown argument [$what]\n";
 		}
 	    }
 	} else {
@@ -493,8 +501,20 @@ sub reload {
 	my $fh = IO::File->new($INC{'CPAN.pm'});
 	local $/;
 	undef $/;
+	$redef = 0;
+	local($SIG{__WARN__})
+	    = sub {
+		if ( $_[0] =~ /Subroutine \w+ redefined/ ) {
+		    ++$redef;
+		    local($|) = 1;
+		    print ".";
+		    return;
+		}
+		warn @_;
+	    };
 	eval <$fh>;
 	warn $@ if $@;
+	print "\n$redef subroutines redefined\n";
     } elsif ($_[1] =~ /index/) {
 	CPAN::Index->force_reload;
     }
@@ -510,10 +530,12 @@ sub _binary_extensions {
 	next if $file =~ /^Contact Author/;
 	next if $file =~ /perl5[._-]\d{3}(?:[\d_]+)?\.tar[._-]gz$/;
 	next unless $module->xs_file;
+	local($|) = 1;
+	print ".";
 	push @result, $module;
     }
 #    print join " | ", @result;
-#    print "\n";
+    print "\n";
     return @result;
 }
 
@@ -689,7 +711,7 @@ sub expand {
 	    }
 	    if ($CPAN::META->exists($class,$xarg)) {
 		$obj = $CPAN::META->instance($class,$xarg);
-	    } elsif ($obj = $CPAN::META->exists($class,$arg)) {
+	    } elsif ($CPAN::META->exists($class,$arg)) {
 		$obj = $CPAN::META->instance($class,$arg);
 	    } else {
 		next;
@@ -737,6 +759,9 @@ sub rematein {
 	    CPAN->debug(qq{pragma[$pragma] meth[$meth] obj[$obj] as_string\[}.$obj->as_string.qq{\]}) if $CPAN::DEBUG;
 	    $obj->$pragma() if $pragma && $obj->can($pragma);
 	    $obj->$meth();
+	} elsif ($CPAN::META->exists('CPAN::Author',$s)) {
+	    $obj = $CPAN::META->instance('CPAN::Author',$s);
+	    print "Don't be silly, you can't $meth ", $obj->fullname, " ;-)\n";
 	} else {
 	    print "Warning: Cannot $meth $s, don't know what it is\n";
 	}
@@ -745,16 +770,20 @@ sub rematein {
 
 #-> sub CPAN::Shell::force ;
 sub force   { shift->rematein('force',@_); }
+#-> sub CPAN::Shell::get ;
+sub get     { shift->rematein('get',@_); }
 #-> sub CPAN::Shell::readme ;
 sub readme  { shift->rematein('readme',@_); }
 #-> sub CPAN::Shell::make ;
 sub make    { shift->rematein('make',@_); }
-#-> sub CPAN::Shell::clean ;
-sub clean   { shift->rematein('clean',@_); }
 #-> sub CPAN::Shell::test ;
 sub test    { shift->rematein('test',@_); }
 #-> sub CPAN::Shell::install ;
 sub install { shift->rematein('install',@_); }
+#-> sub CPAN::Shell::clean ;
+sub clean   { shift->rematein('clean',@_); }
+#-> sub CPAN::Shell::look ;
+sub look   { shift->rematein('look',@_); }
 
 package CPAN::FTP;
 use vars qw($Ua);
@@ -768,6 +797,7 @@ sub ftp_get {
 	on host [$host] as local [$target]\n]
 		      ) if $CPAN::DEBUG;
     my $ftp = Net::FTP->new($host);
+    return 0 unless defined $ftp;
     $ftp->debug(1) if $CPAN::DEBUG{'FTP'} & $CPAN::DEBUG;
     $class->debug(qq[Going to ->login("anonymous","$Config::Config{'cf_email'}")\n]);
     unless ( $ftp->login("anonymous",$Config::Config{'cf_email'}) ){
@@ -808,9 +838,13 @@ sub localize {
 	require LWP::UserAgent;
  	unless ($Ua) {
 	    $Ua = new LWP::UserAgent;
-	    $Ua->proxy('ftp',  $ENV{'ftp_proxy'})  if defined $ENV{'ftp_proxy'};
-	    $Ua->proxy('http', $ENV{'http_proxy'}) if defined $ENV{'http_proxy'};
-	    $Ua->no_proxy($ENV{'no_proxy'})        if defined $ENV{'no_proxy'};
+	    my($var);
+	    $Ua->proxy('ftp',  $var)
+		if $var = $CPAN::Config->{'ftp_proxy'} || $ENV{'ftp_proxy'};
+	    $Ua->proxy('http', $var)
+		if $var = $CPAN::Config->{'http_proxy'} || $ENV{'http_proxy'};
+	    $Ua->no_proxy($var)
+		if $var = $CPAN::Config->{'no_proxy'} || $ENV{'no_proxy'};
 	}
     }
 
@@ -839,23 +873,107 @@ sub localize {
 	}
 
 	if ($CPAN::META->hasLWP) {
-	    print "Fetching $url\n";
+	    print "Fetching $url with LWP\n";
 	    my $res = $Ua->mirror($url, $aslocal);
 	    if ($res->is_success) {
 		return $aslocal;
 	    }
 	}
-	if ($CPAN::META->hasFTP && $url =~ m|^ftp://(.*?)/(.*)/(.*)|) {
+	if ($url =~ m|^ftp://(.*?)/(.*)/(.*)|) {
 	    my($host,$dir,$getfile) = ($1,$2,$3);
-	    $dir =~ s|/+|/|g;
-	    $self->debug("Going to fetch file [$getfile]
+	    if ($CPAN::META->hasFTP) {
+		$dir =~ s|/+|/|g;
+		$self->debug("Going to fetch file [$getfile]
   from dir [$dir]
   on host  [$host]
   as local [$aslocal]") if $CPAN::DEBUG;
-	    CPAN::FTP->ftp_get($host,$dir,$getfile,$aslocal) && return $aslocal;
+		CPAN::FTP->ftp_get($host,$dir,$getfile,$aslocal) && return $aslocal;
+	    } 
+	    # Came back if Net::FTP couldn't establish connection...
+	    # Maybe they are behind a firewall, but they gave us
+	    # a socksified ftp program...
+	    if (-x $CPAN::Config->{'ftp'}) {
+		my($netrc) = CPAN::FTP::netrc->new;
+		use File::stat;
+		my $timestamp = 0;
+		my $st = stat($aslocal);
+		$timestamp = $st->mtime if defined $st;
+		if (
+		    ($netrc->hasdefault() || $netrc->contains($host)) &&
+		    $netrc->protected
+		) {
+		    print(
+			  qq{
+  Trying with external ftp to get $url
+  As this requires some features that are not thoroughly tested, we\'re
+  not sure, that we get it right. Please, install Net::FTP as soon
+  as possible. Just type "install Net::FTP". Thank you.
+
+}
+			 );
+		    my($fh) = IO::File->new;
+		    my($targetfile) = File::Basename::basename($aslocal);
+		    my(@dialog);
+		    push @dialog, "lcd $aslocal_dir";
+		    push @dialog, map {"cd $_"} split "/", $dir; # RFC 1738
+		    push @dialog, "get $getfile $targetfile";
+		    push @dialog, "quit";
+		    open($fh, "|$CPAN::Config->{'ftp'} $host") or die "Couldn't open ftp: $!";
+		    # pilot is blind now
+		    foreach (@dialog) {
+			$fh->print("$_\n");
+		    }
+		    close $fh;		# Wait for process to complete
+		    $st = stat($aslocal);
+		    if (defined $st && $st->mtime > $timestamp) {
+			print "GOT $aslocal\n";
+			return $aslocal;
+		    } else {
+			print "Hmm... Still failed!\n";
+		    }
+		} else {
+		    my($netrcfile) = $netrc->netrc();
+		    if ($netrcfile){
+			print "Your $netrcfile does not contain host $host.\n"
+			    if $netrc->protected;
+			print "Your $netrcfile is not correctly protected.\n"
+			    unless $netrc->protected;
+ 		    } else {
+			print qq{  I could not find or open your .netrc file.\n}
+		    }
+		}
+		# OK, they don't have a valid ~/.netrc. Use 'ftp -n' then and
+		# login manually to host, using e-mail as password.
+		print "Issuing ftp the hard way...\n";
+		my $targetfile = File::Basename::basename($aslocal);
+		my $cd_to_dir = map {"cd $_\n"} split "/", $dir; # RFC 1738
+		my $template =
+"open $host
+ user anonymous $Config::Config{'cf_email'}
+ bin
+ lcd $aslocal_dir
+ $cd_to_dir
+ get $getfile $targetfile
+ bye
+";
+		my $cmd = IO::File->new;
+		$cmd->open("|$CPAN::Config->{'ftp'} -n") or
+		    die "Cannot fork: $!\n";
+		$cmd->print($template);
+		$cmd->close;
+		$st = stat($aslocal);
+		if (defined $st && $st->mtime > $timestamp) {
+			print "GOT $aslocal\n";
+			return $aslocal;
+		} else {
+			print "Bad luck... Still failed!\n";
+		}
+	    }
+	    sleep 2;
 	}
 	if (-x $CPAN::Config->{'lynx'}) {
 ##	    $self->debug("Trying with lynx for [$url]") if $CPAN::DEBUG;
+	    my($want_compressed);
 	    print(
 		  qq{
   Trying with lynx to get $url
@@ -865,17 +983,30 @@ sub localize {
 
 }
 		 );
-	    $aslocal =~ s/\.gz//;
+	    $want_compressed = $aslocal =~ s/\.gz//;
 	    my($system) = "$CPAN::Config->{'lynx'} -source '$url' > $aslocal";
 	    if (system($system)==0) {
-		$system = "$CPAN::Config->{'gzip'} $aslocal";
-		system($system)==0 or die;
-		return "$aslocal.gz";
+		if ($want_compressed) {
+		    $system = "$CPAN::Config->{'gzip'} -dt $aslocal";
+		    if (system($system)==0) {
+			rename $aslocal, "$aslocal.gz";
+		    } else {
+			$system = "$CPAN::Config->{'gzip'} $aslocal";
+			system($system);
+		    }
+		    return "$aslocal.gz";
+		} else {
+		    $system = "$CPAN::Config->{'gzip'} -dt $aslocal";
+		    if (system($system)==0) {
+			$system = "$CPAN::Config->{'gzip'} -d $aslocal";
+			system($system);
+		    } else {
+			# should be fine, eh?
+		    }
+		    return $aslocal;
+		}
 	    }
 	}
-	if (-x $CPAN::Config->{'ftp'}) {
-	    print qq{The external ftp is not yet supported, sorry\n};
-	}
 	warn "Can't access URL $url.
   Either get LWP or Net::FTP
   or an external lynx or ftp";
@@ -883,6 +1014,53 @@ sub localize {
     Carp::croak("Cannot fetch $file from anywhere");
 }
 
+package CPAN::FTP::external;
+
+package CPAN::FTP::netrc;
+
+sub new {
+    my($class) = @_;
+    my $file = MY->catfile($ENV{HOME},".netrc");
+    my($fh,@machines,$hasdefault);
+    $hasdefault = 0;
+    if($fh = IO::File->new($file,"r")){
+	local($/) = "";
+      NETRC: while (<$fh>) {
+	    my(@tokens) = split ' ', $_;
+	  TOKEN: while (@tokens) {
+		my($t) = shift @tokens;
+		$hasdefault++, last NETRC if $t eq "default"; # we will most
+                                                        # probably be
+                                                        # able to anonftp
+		last TOKEN if $t eq "macdef";
+		if ($t eq "machine") {
+		    push @machines, shift @tokens;
+		}
+	    }
+	}
+    } else {
+	$file = "";
+    }
+    use File::stat;
+    my $st = stat($file);
+    my $protected = 0;
+    $protected = ($st->mode & 077) == 0 if defined $st;
+    bless {
+	   'mach' => [@machines],
+	   'netrc' => $file,
+	   'hasdefault' => $hasdefault,
+	   'protected' => $protected,
+	  }, $class;
+}
+
+sub hasdefault { shift->{'hasdefault'} }
+sub netrc { shift->{'netrc'} }
+sub protected { shift->{'protected'} }
+sub contains {
+    my($self,$mach) = @_;
+    scalar grep {$_ eq $mach} @{$self->{'mach'}};
+}
+
 package CPAN::Complete;
 @CPAN::Complete::ISA = qw(CPAN::Debug);
 
@@ -894,10 +1072,19 @@ sub complete {
     $pos ||= 0;
     CPAN->debug("word [$word] line[$line] pos[$pos]") if $CPAN::DEBUG;
     $line =~ s/^\s*//;
+    if ($line =~ s/^(force\s*)//) {
+	$pos -= length($1);
+    }
     my @return;
     if ($pos == 0) {
-	@return = grep(/^$word/, sort qw(! a b d h i m o q r u autobundle clean make test install reload));
-    } elsif ( $line !~ /^[\!abdhimorut]/ ) {
+	@return = grep(
+		       /^$word/,
+		       sort qw(
+			       ! a b d h i m o q r u autobundle clean
+			       make test install force reload look
+			      )
+		      );
+    } elsif ( $line !~ /^[\!abdhimorutl]/ ) {
 	@return = ();
     } elsif ($line =~ /^a\s/) {
 	@return = completex('CPAN::Author',$word);
@@ -905,7 +1092,7 @@ sub complete {
 	@return = completex('CPAN::Bundle',$word);
     } elsif ($line =~ /^d\s/) {
 	@return = completex('CPAN::Distribution',$word);
-    } elsif ($line =~ /^([mru]\s|(make|clean|test|install)\s)/ ) {
+    } elsif ($line =~ /^([mru]|make|clean|test|install|readme|look)\s/ ) {
 	@return = (completex('CPAN::Module',$word),completex('CPAN::Bundle',$word));
     } elsif ($line =~ /^i\s/) {
 	@return = complete_any($word);
@@ -1236,6 +1423,9 @@ sub get {
     my $packagedir;
 
     $self->debug("local_file[$local_file]") if $CPAN::DEBUG;
+    if ($CPAN::META->hasMD5) {
+	$self->verifyMD5;
+    }
     if ($local_file =~ /(\.tar\.(gz|Z)|\.tgz|\.zip)$/i){
 	$self->debug("Removing tmp") if $CPAN::DEBUG;
 	File::Path::rmtree("tmp");
@@ -1324,10 +1514,51 @@ sub new {
     return bless $this, $class;
 }
 
+#-> sub CPAN::Distribution::look ;
+sub look {
+    my($self) = @_;
+    if (  $CPAN::Config->{'shell'} ) {
+	print qq{
+Trying to open a subshell in the build directory...
+};
+    } else {
+	print qq{
+Your configuration does not define a value for subshells.
+Please define it with "o conf shell <your shell>"
+};
+	return;
+    }
+    my $dist = $self->id;
+    my $dir  = $self->dir or $self->get;
+    $dir = $self->dir;
+    my $pwd  = Cwd::cwd();
+    chdir($dir);
+    print qq{Working directory is $dir.\n};
+    system($CPAN::Config->{'shell'})==0 or die "Subprocess shell error";
+    chdir($pwd);
+}
+
 #-> sub CPAN::Distribution::readme ;
 sub readme {
     my($self) = @_;
-    print "Readme not yet implemented (says ".$self->id.")\n";
+    my($dist) = $self->id;
+    my($sans,$suffix) = $dist =~ /(.+)\.(tgz|tar[\._-]gz|tar\.Z|zip)$/;
+    $self->debug("sans[$sans] suffix[$suffix]\n") if $CPAN::DEBUG;
+    my($local_file);
+    my($local_wanted) =
+	 CPAN->catfile(
+			$CPAN::Config->{keep_source_where},
+			"authors",
+			"id",
+			split("/","$sans.readme"),
+		       );
+    $self->debug("Doing localize") if $CPAN::DEBUG;
+    $local_file = CPAN::FTP->localize("authors/id/$sans.readme", $local_wanted);
+    my $fh_pager = IO::File->new;
+    $fh_pager->open("|$CPAN::Config->{'pager'}") or die "Could not open pager $CPAN::Config->{'pager'}: $!";
+    my $fh_readme = IO::File->new;
+    $fh_readme->open($local_file) or die "Could not open $local_file: $!";
+    $fh_pager->print(<$fh_readme>);
 }
 
 #-> sub CPAN::Distribution::verifyMD5 ;
@@ -1446,9 +1677,6 @@ sub make {
     $self->debug($self->id) if $CPAN::DEBUG;
     print "Running make\n";
     $self->get;
-    if ($CPAN::META->hasMD5) {
-	$self->verifyMD5;
-    }
     EXCUSE: {
 	  my @e;
 	  $self->{archived} eq "NO" and push @e, "Is neither a tar nor a zip archive.";
@@ -1457,7 +1685,7 @@ sub make {
 	  defined $self->{'make'} and push @e, "Has already been processed within this session";
 	  print join "", map {"  $_\n"} @e and return if @e;
      }
-    print "\n  CPAN: Going to build ".$self->id."\n\n";
+    print "\n  CPAN.pm: Going to build ".$self->id."\n\n";
     my $builddir = $self->dir;
     chdir $builddir or Carp::croak("Couldn't chdir $builddir: $!");
     $self->debug("Changed directory to $builddir") if $CPAN::DEBUG;
@@ -1698,23 +1926,23 @@ sub rematein {
 
 #-> sub CPAN::Bundle::force ;
 sub force   { shift->rematein('force',@_); }
+#-> sub CPAN::Bundle::get ;
+sub get     { shift->rematein('get',@_); }
+#-> sub CPAN::Bundle::make ;
+sub make    { shift->rematein('make',@_); }
+#-> sub CPAN::Bundle::test ;
+sub test    { shift->rematein('test',@_); }
 #-> sub CPAN::Bundle::install ;
 sub install { shift->rematein('install',@_); }
 #-> sub CPAN::Bundle::clean ;
 sub clean   { shift->rematein('clean',@_); }
-#-> sub CPAN::Bundle::test ;
-sub test    { shift->rematein('test',@_); }
-#-> sub CPAN::Bundle::make ;
-sub make    { shift->rematein('make',@_); }
 
-# XXX not yet implemented!
 #-> sub CPAN::Bundle::readme ;
 sub readme  {
     my($self) = @_;
     my($file) = $self->cpan_file or print("No File found for bundle ", $self->id, "\n"), return;
     $self->debug("self[$self] file[$file]") if $CPAN::DEBUG;
     $CPAN::META->instance('CPAN::Distribution',$file)->readme;
-#    CPAN::FTP->localize("authors/id/$file",$index_wanted); # XXX
 }
 
 package CPAN::Module;
@@ -1841,10 +2069,12 @@ sub rematein {
 
 #-> sub CPAN::Module::readme ;
 sub readme { shift->rematein('readme') }
+#-> sub CPAN::Module::look ;
+sub look { shift->rematein('look') }
+#-> sub CPAN::Module::get ;
+sub get    { shift->rematein('get',@_); }
 #-> sub CPAN::Module::make ;
 sub make   { shift->rematein('make') }
-#-> sub CPAN::Module::clean ;
-sub clean  { shift->rematein('clean') }
 #-> sub CPAN::Module::test ;
 sub test   { shift->rematein('test') }
 #-> sub CPAN::Module::install ;
@@ -1865,6 +2095,8 @@ sub install {
     }
     $self->rematein('install') if $doit;
 }
+#-> sub CPAN::Module::clean ;
+sub clean  { shift->rematein('clean') }
 
 #-> sub CPAN::Module::inst_file ;
 sub inst_file {
@@ -2275,9 +2507,9 @@ Batch mode:
 =head1 DESCRIPTION
 
 The CPAN module is designed to automate the make and install of perl
-modules and extensions. It includes some searching capabilities as
-well knows a how to use Net::FTP or LWP to fetch the raw data from the
-net.
+modules and extensions. It includes some searching capabilities and
+knows how to use Net::FTP or LWP (or lynx or an external ftp client)
+to fetch the raw data from the net.
 
 Modules are fetched from one or more of the mirrored CPAN
 (Comprehensive Perl Archive Network) sites and unpacked in a dedicated
@@ -2291,7 +2523,8 @@ The package contains a session manager and a cache manager. There is
 no status retained between sessions. The session manager keeps track
 of what has been fetched, built and installed in the current
 session. The cache manager keeps track of the disk space occupied by
-the make processes and deletes excess space in a simple FIFO style.
+the make processes and deletes excess space according to a simple FIFO
+mechanism.
 
 All methods provided are accessible in a programmer style and in an
 interactive shell style.
@@ -2316,9 +2549,9 @@ The most common uses of the interactive modes are
 =item Searching for authors, bundles, distribution files and modules
 
 There are corresponding one-letter commands C<a>, C<b>, C<d>, and C<m>
-for each of the four categories and another, C<i> for any of the other
-four. Each of the four entities is implemented as a class with
-slightly differing methods for displaying an object.
+for each of the four categories and another, C<i> for any of the
+mentioned four. Each of the four entities is implemented as a class
+with slightly differing methods for displaying an object.
 
 Arguments you pass to these commands are either strings matching exact
 the identification string of an object or regular expressions that are
@@ -2347,19 +2580,24 @@ each as object-E<gt>as_glimpse. E.g.
     Author          ANDYD (Andy Dougherty)
     Author          MERLYN (Randal L. Schwartz)
 
-=item make, test, install, clean modules or distributions
+=item make, test, install, clean  modules or distributions
 
-The four commands do indeed exist just as written above. Each of them
-takes as many arguments as provided and investigates for each what it
-might be. Is it a distribution file (recognized by embedded slashes),
-this file is being processed. Is it a module, CPAN determines the
+These commands do indeed exist just as written above. Each of them
+takes any number of arguments and investigates for each what it might
+be. Is it a distribution file (recognized by embedded slashes), this
+file is being processed. Is it a module, CPAN determines the
 distribution file where this module is included and processes that.
 
-Any C<make> and C<test> are run unconditionally. An C<install
-E<lt>distribution_fileE<gt>> also is run unconditionally.  But for
-C<install E<lt>module<gt>> CPAN checks if an install is actually
-needed for it and prints I<"Foo up to date"> in case the module
-doesnE<39>t need to be updated.
+Any C<make>, C<test>, and C<readme> are run unconditionally. A 
+
+  C<install E<lt>distribution_fileE<gt>>
+
+also is run unconditionally.  But for 
+
+  C<install E<lt>moduleE<gt>>
+
+CPAN checks if an install is actually needed for it and prints
+I<Foo up to date> in case the module doesnE<39>t need to be updated.
 
 CPAN also keeps track of what it has done within the current session
 and doesnE<39>t try to build a package a second time regardless if it
@@ -2377,6 +2615,14 @@ Example:
     OpenGL-0.4/COPYRIGHT
     [...]
 
+=item readme, look module or distribution
+
+These two commands take only one argument, be it a module or a
+distribution file. C<readme> displays the README of the associated
+distribution file. C<Look> gets and untars (if not yet done) the
+distribution file, changes to the appropriate directory and opens a
+subshell process in that directory.
+
 =back
 
 =head2 CPAN::Shell
@@ -2388,19 +2634,47 @@ acts like most shells do. The first word is being interpreted as the
 method to be called and the rest of the words are treated as arguments
 to this method.
 
+=head2 autobundle
+
+C<autobundle> writes a bundle file into the
+C<$CPAN::Config-E<gt>{cpan_home}/Bundle> directory. The file contains
+a list of all modules that are both available from CPAN and currently
+installed within @INC. The name of the bundle file is based on the
+current date and a counter.
+
+=head2 recompile
+
+recompile() is a very special command in that it takes no argument and
+runs the make/test/install cycle with brute force over all installed
+dynamically loadable extensions (aka XS modules) with 'force' in
+effect. Primary purpose of this command is to finish a network
+installation. Imagine, you have a common source tree for two different
+architectures. You decide to do a completely independent fresh
+installation. You start on one architecture with the help of a Bundle
+file produced earlier. CPAN installs the whole Bundle for you, but
+when you try to repeat the job on the second architecture, CPAN
+responds with a C<"Foo up to date"> message for all modules. So you
+will be glad to run recompile in the second architecture and
+youE<39>re done.
+
+Another popular use for C<recompile> is to act as a rescue in case your
+perl breaks binary compatibility. If one of the modules that CPAN uses
+is in turn depending on binary compatibility (so you cannot run CPAN
+commands), then you should try the CPAN::Nox module for recovery.
+
 =head2 ProgrammerE<39>s interface
 
 If you do not enter the shell, the available shell commands are both
 available as methods (C<CPAN::Shell-E<gt>install(...)>) and as
 functions in the calling package (C<install(...)>). The
 programmerE<39>s interface has beta status. Do not heavily rely on it,
-changes may still happen.
+changes may still be necessary.
 
 =head2 Cache Manager
 
 Currently the cache manager only keeps track of the build directory
 ($CPAN::Config->{build_dir}). It is a simple FIFO mechanism that
-deletes complete directories below build_dir as soon as the size of
+deletes complete directories below C<build_dir> as soon as the size of
 all directories there gets bigger than $CPAN::Config->{build_cache}
 (in MB). The contents of this cache may be used for later
 re-installations that you intend to do manually, but will never be
@@ -2436,9 +2710,7 @@ of the line is optional. The comment part is delimited by a dash just
 as in the man page header.
 
 The distribution of a bundle should follow the same convention as
-other distributions. The bundle() function in the CPAN module simply
-parses the module that defines the bundle and returns the module names
-that are listed in the described CONTENTS section.
+other distributions.
 
 Bundles are treated specially in the CPAN package. If you say 'install
 Bundle::Tkkit' (assuming such a bundle exists), CPAN will install all
@@ -2452,33 +2724,68 @@ There is a meaningless Bundle::Demo available on CPAN. Try to install
 it, it usually does no harm, just demonstrates what the Bundle
 interface looks like.
 
-=head2 autobundle
+=head2 Prerequisites
 
-C<autobundle> writes a bundle file into the 
-C<$CPAN::Config->{cpan_home}/Bundle> directory. The file contains a list
-of all modules that are both available from CPAN and currently
-installed within @INC. The name of the bundle file is based on the
-current date and a counter.
+If you have a local mirror of CPAN and can access all files with
+"file:" URLs, then you only need a perl better than perl5.003 to run
+this module. Otherwise Net::FTP is strongly recommended. LWP may be
+required for non-UNIX systems or if your nearest CPAN site is
+associated with an URL that is not C<ftp:>.
 
-=head2 recompile
+If you have neither Net::FTP nor LWP, there is a fallback mechanism
+implemented for an external ftp command or for an external lynx
+command.
 
-recompile() is a very special command in that it takes no argument and
-runs the make/test/install cycle with brute force over all installed
-dynamically loadable extensions (aka XS modules) with 'force' in
-effect. Primary purpose of this command is to act as a rescue in case
-your perl breaks binary compatibility. If one of the modules that CPAN
-uses is in turn depending on binary compatibility (so you cannot run
-CPAN commands), then you should try the CPAN::Nox module for recovery.
+This module presumes that all packages on CPAN
 
-A very popular use for recompile is to finish a network
-installation. Imagine, you have a common source tree for two different
-architectures. You decide to do a completely independent fresh
-installation. You start on one architecture with the help of a Bundle
-file produced earlier. CPAN installs the whole Bundle for you, but
-when you try to repeat the job on the second architecture, CPAN
-responds with a C<"Foo up to date"> message for all modules. So you
-will be glad to run recompile in the second architecture and
-youE<39>re done.
+=over 2
+
+=item *
+
+declare their $VERSION variable in an easy to parse manner. This
+prerequisite can hardly be relaxed because it consumes by far too much
+memory to load all packages into the running program just to determine
+the $VERSION variable . Currently all programs that are dealing with
+version use something like this
+
+    perl -MExtUtils::MakeMaker -le \
+        'print MM->parse_version($ARGV[0])' filename
+
+If you are author of a package and wonder if your $VERSION can be
+parsed, please try the above method.
+
+=item *
+
+come as compressed or gzipped tarfiles or as zip files and contain a
+Makefile.PL (well we try to handle a bit more, but without much
+enthusiasm).
+
+=back
+
+=head2 Debugging
+
+The debugging of this module is pretty difficult, because we have
+interferences of the software producing the indices on CPAN, of the
+mirroring process on CPAN, of packaging, of configuration, of
+synchronicity, and of bugs within CPAN.pm.
+
+In interactive mode you can try "o debug" which will list options for
+debugging the various parts of the package. The output may not be very
+useful for you as it's just a byproduct of my own testing, but if you
+have an idea which part of the package may have a bug, it's sometimes
+worth to give it a try and send me more specific output. You should
+know that "o debug" has built-in completion support.
+
+=head2 Floppy, Zip, and all that Jazz
+
+CPAN.pm works nicely without network too. If you maintain machines
+that are not networked at all, you should consider working with file:
+URLs. Of course, you have to collect your modules somewhere first. So
+you might use CPAN.pm to put together all you need on a networked
+machine. Then copy the $CPAN::Config->{keep_source_where} (but not
+$CPAN::Config->{build_dir}) directory on a floppy. This floppy is kind
+of a personal CPAN. CPAN.pm on the non-networked machines works nicely
+with this floppy.
 
 =head1 CONFIGURATION
 
@@ -2492,23 +2799,25 @@ use() or require() statements.
 Currently the following keys in the hash reference $CPAN::Config are
 defined:
 
-  build_cache       size of cache for directories to build modules
-  build_dir         locally accessible directory to build modules
-  index_expire      after how many days refetch index files
-  cpan_home         local directory reserved for this package
-  gzip		    location of external program gzip
+  build_cache        size of cache for directories to build modules
+  build_dir          locally accessible directory to build modules
+  index_expire       after how many days refetch index files
+  cpan_home          local directory reserved for this package
+  gzip		     location of external program gzip
+  inactivity_timeout breaks interactive Makefile.PLs after that
+                     many seconds inactivity. Set to 0 to never break.
   inhibit_startup_message
-                    if true, does not print the startup message
-  keep_source       keep the source in a local directory?
-  keep_source_where where keep the source (if we do)
-  make              location of external program make
-  make_arg	    arguments that should always be passed to 'make'
-  make_install_arg  same as make_arg for 'make install'
-  makepl_arg	    arguments passed to 'perl Makefile.PL'
-  pager             location of external program more (or any pager)
-  tar               location of external program tar
-  unzip             location of external program unzip
-  urllist	    arrayref to nearby CPAN sites (or equivalent locations)
+                     if true, does not print the startup message
+  keep_source        keep the source in a local directory?
+  keep_source_where  where keep the source (if we do)
+  make               location of external program make
+  make_arg	     arguments that should always be passed to 'make'
+  make_install_arg   same as make_arg for 'make install'
+  makepl_arg	     arguments passed to 'perl Makefile.PL'
+  pager              location of external program more (or any pager)
+  tar                location of external program tar
+  unzip              location of external program unzip
+  urllist	     arrayref to nearby CPAN sites (or equivalent locations)
 
 You can set and query each of these options interactively in the cpan
 shell with the command set defined within the C<o conf> command:
@@ -2534,8 +2843,7 @@ shifts or pops the array in the I<list option> variable
 
 =item o conf E<lt>list optionE<gt> [unshift|push|splice] E<lt>listE<gt>
 
-works like the corresponding perl commands. Whitespace is used to
-determine the arguments.
+works like the corresponding perl commands.
 
 =back
 
@@ -2546,7 +2854,7 @@ install foreign, unmasked, unsigned code on your machine. We compare
 to a checksum that comes from the net just as the distribution file
 itself. If somebody has managed to tamper with the distribution file,
 they may have as well tampered with the CHECKSUMS file. Future
-development will go towards stong authentification.
+development will go towards strong authentification.
 
 =head1 EXPORT
 
@@ -2554,53 +2862,15 @@ Most functions in package CPAN are exported per default. The reason
 for this is that the primary use is intended for the cpan shell or for
 oneliners.
 
-=head1 Debugging
+=head1 BUGS
 
-The debugging of this module is pretty difficult, because we have
-interferences of the software producing the indices on CPAN, of the
-mirroring process on CPAN, of packaging, of configuration, of
-synchronicity, and of bugs within CPAN.pm.
+we should give coverage for _all_ of the CPAN and not just the
+__PAUSE__ part, right? In this discussion CPAN and PAUSE have become
+equal -- but they are not. PAUSE is authors/ and modules/. CPAN is
+PAUSE plus the clpa/, doc/, misc/, ports/, src/, scripts/.
 
-In interactive mode you can try "o debug" which will list options for
-debugging the various parts of the package. The output may not be very
-useful for you as it's just a byproduct of my own testing, but if you
-have an idea which part of the package may have a bug, it's sometimes
-worth to give it a try and send me more specific output. You should
-know that "o debug" has built-in completion support.
-
-=head2 Prerequisites
-
-If you have a local mirror of CPAN and can access all files with
-"file:" URLs, then you only need perl5.003 to run this
-module. Otherwise you need Net::FTP intalled. LWP may be required for
-non-UNIX systems or if your nearest CPAN site is associated with an
-URL that is not C<ftp:>.
-
-This module presumes that all packages on CPAN
-
-=over 2
-
-=item *
-
-declare their $VERSION variable in an easy to parse manner. This
-prerequisite can hardly be relaxed because it consumes by far too much
-memory to load all packages into the running program just to determine
-the $VERSION variable . Currently all programs that are dealing with
-VERSION use something like this
-
-    perl -MExtUtils::MakeMaker -le \
-        'print MM->parse_version($ARGV[0])' filename
-
-If you are author of a package and wonder if your VERSION can be
-parsed, please try the above method.
-
-=item *
-
-come as compressed or gzipped tarfiles or as zip files and contain a
-Makefile.PL (well we try to handle a bit more, but without much
-enthusiasm).
-
-=back
+Future development should be directed towards a better intergration of
+the other parts.
 
 =head1 AUTHOR
 
